# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "centaur"


def test_query(self):
    result = self.decode(
        """
    H|\\^&|||ACCP1|||||Host||P|1
    Q|1|^002230747|^002230747|ALL||||||||O
    L|1"""
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "002230747"}],
    }
    assert result == expected


def test_void_order(self):
    result = self.order(
        worklist=self.void_worklist,
        context={
            "query": self.parse(
                """
                H|\\^&|||ACCP1|||||Host||P|1
                Q|1|^002230747|^002230747|ALL||||||||O
                L|1"""
            )
        },
    )
    expected = """
        H|\\^&|||Host|||||ACCP1||P|1
        L|1|I"""
    assert result == expected


def test_order(self):
    result = self.order()
    expected = """H|\\^&|||Host|||||ACCP1||P|1
        P|1|036666666|||NYME^ANO^NJF||20110513||||||||||||||||||LOC
        O|1|123456||^^^A\\^^^B|R||||||||||||||||||||O
        O|2|123456||^^^C|R||||||C||||||||||||||O
        L|1|F"""
    assert result == expected


def test_result(self):
    result = self.decode(
        """
    H|\\^&|||ACCP1|||||Host||P|1
    P|1|013199|||TEST^ANONYME||19900212|U
    O|1|01899999||^^^aHBs2|R||||||||||||||||||||F
    R|1|^^^aHBs2^^^1^INTR|React|||>||F||||20131125141811
    C|1|I|Above Check|I
    R|2|^^^aHBs2^^^1^DOSE|> 1000.00|mIU/mL||>||F||||20131125141811
    C|1|I|Above Check|I
    R|3|^^^aHBs2^^^1^COFF|1.00|mIU/mL||>||F||||20131125141811
    C|1|I|Above Check|I
    R|4|^^^aHBs2^^^1^RLU|4663906|||>||F||||20131125141811
    C|1|I|Above Check|I
    L|1"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": None,
                "sample_id": "01899999",
                "type": "patient",
                "results": [
                    {
                        "flags": [">", "Above Check"],
                        "lots": [],
                        "comments": [],
                        "date": datetime.datetime(2013, 11, 25, 14, 18, 11),
                        "value": "React",
                        "code": "aHBs2INTR",
                    },
                    {
                        "flags": [">", "Above Check"],
                        "comments": [],
                        "lots": [],
                        "date": datetime.datetime(2013, 11, 25, 14, 18, 11),
                        "value": "> 1000.00",
                        "code": "aHBs2DOSE",
                    },
                    {
                        "flags": [">", "Above Check"],
                        "comments": [],
                        "lots": [],
                        "date": datetime.datetime(2013, 11, 25, 14, 18, 11),
                        "value": "1.00",
                        "code": "aHBs2COFF",
                    },
                    {
                        "flags": [">", "Above Check"],
                        "comments": [],
                        "lots": [],
                        "date": datetime.datetime(2013, 11, 25, 14, 18, 11),
                        "value": "4663906",
                        "code": "aHBs2RLU",
                    },
                ],
            }
        ],
    }
    assert result == expected
