# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "cs2x00i"


def test_query(self):
    result = self.decode(
        """
    H|\\^&|||CS-2000i^^10000001^^^CANEXT^06317410||||||||E1394-97
    Q|1|000001^01^ 1^B||^^^040^PT\\^^^060^Fbg\\^^^120^II\\^^^150^V\\^^^170^VII\\^^^180^VIII\\^^^190^IX\\^^^200^X\\^^^210^XI\\^^^300^AT3\\^^^310^APL|0|20070328133318
    L|1|N"""
    )
    expected = {"context": "query", "samples": [{"sample_id": "1", "type": "patient"}]}
    assert result == expected


def test_result(self):
    result = self.decode(
        """
    H|\\^&|||CS-2000i^00-30^10000001^^^CANEXT^06317410||||||||E1394-97
    P|1||||^Thomas^Johnson
    O|1||000001^01^ 1^B^||R||||||N
    R|1|^^^041^PT sec^100.00^9^^^|  10.2|sec||<^[0000.6000.0200 Limite de rendu inférieure dépassée]^||||||20070328135056
    R|2|^^^062^FIB^50.00^9^^D^|  9.00|g/L||>^[1100.4000.0000 Volume d'échantillon erroné],[0000.6000.0100 Limite de rendu supérieur dépassée]^||||||20070328135056
    R|2|^^^^Lipemic Sample^^^^^||||A||||||20130523113045
    R|3|^^^^Icterous Sample^^^^^||||A||||||20130523113045
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "1",
                "rack": "000001-01",
                "type": "patient",
                "results": [
                    {
                        "code": "041",
                        "value": "10.2",
                        "dilution": "100.00",
                        "date": datetime.datetime(2007, 3, 28, 13, 50, 56),
                        "flags": ["<", "0000.6000.0200"],
                    },
                    {
                        "code": "062",
                        "value": "9.00",
                        "dilution": "50.00",
                        "date": datetime.datetime(2007, 3, 28, 13, 50, 56),
                        "flags": [">", "1100.4000.0000", "0000.6000.0100"],
                    },
                    {
                        "code": "L",
                        "value": "YES",
                        "dilution": "50.00",
                        "date": datetime.datetime(2013, 5, 23, 11, 30, 45),
                        "flags": ["A"],
                    },
                    {
                        "code": "I",
                        "value": "YES",
                        "dilution": "50.00",
                        "date": datetime.datetime(2013, 5, 23, 11, 30, 45),
                        "flags": ["A"],
                    },
                ],
            }
        ],
    }
    assert result == expected


def test_result_qc(self):
    result = self.decode(
        """
    H|\\^&|||CS-5100^01-18^24038^^^CS-5100^BY990757||||||||E1394-97
    P|1||||
    O|1||REAG00^  ^QCIHepLMW553322^M^||R||||||Q
    R|1|^^^342^INN Hep~IU_mL^100.00^A^^^|  0.41|IU/mL||N^^||||||20190527094643
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "QCIHepLMW553322",
                "sample_id": "QCIHepLMW553322",
                "results": [
                    {
                        "code": "342",
                        "date": datetime.datetime(2019, 5, 27, 9, 46, 43),
                        "dilution": "100.00",
                        "flags": ["N"],
                        "value": "0.41",
                    }
                ],
                "type": "control",
            }
        ],
    }
    assert result == expected


def test_result_ignore_defective_volume(self):
    result = self.decode(
        """
    H|\\^&|||CS-2500^01-68^21526^^^CP^BV981798||||||||E1394-97
    P|1||||^TEST              ^
    O|1||000012^01^     1014017026^B^||R||||||N
    R|1|^^^041^TQ^100.00^9^^^|  21.1|sec||N^[1100.4000.0000 Volume d'échantillon erroné]^||||||20180314114007
    R|2|^^^042^TP^100.00^9^^^|  40.8|%||N^[1100.4000.0000 Volume d'échantillon erroné]^||||||20180314114007
    R|5|^^^^Defective Sample Volume^^^^^||||N||||||20180314114007
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "1014017026",
                "rack": "000012-01",
                "type": "patient",
                "results": [
                    {
                        "code": "041",
                        "value": "21.1",
                        "dilution": "100.00",
                        "flags": ["N", "1100.4000.0000"],
                        "date": datetime.datetime(2018, 3, 14, 11, 40, 7),
                    },
                    {
                        "code": "042",
                        "value": "40.8",
                        "dilution": "100.00",
                        "flags": ["N", "1100.4000.0000"],
                        "date": datetime.datetime(2018, 3, 14, 11, 40, 7),
                    },
                ],
            }
        ],
    }
    assert result == expected


def test_void_query(self):
    result = self.order(
        worklist=self.void_worklist,
        context={
            "query": self.parse(
                """
                H|\\^&|||CS-2000i^^10000001^^^CANEXT^06317410||||||||E1394-97
                Q|1|000001^01^ 1^B||^^^040^PT\\^^^060^Fbg\\^^^120^II\\^^^150^V\\^^^170^VII\\^^^180^VIII\\^^^190^IX\\^^^200^X\\^^^210^XI\\^^^300^AT3\\^^^310^APL|0|20070328133318
                L|1|N"""
            )
        },
    )
    expected = """
    H|\\^&|||||||||||E1394-97|20130715141912
    P|1|||||||U
    O|1|000001^01^ 1^B
    L|1|N"""
    assert result == expected


def test_order(self):
    result = self.order()
    expected = """H|\\^&|||||||||||E1394-97|20130715141912
        P|1|||036666666|^ANO^NYME||20110513|F|||||||||||||||||^^^LOC
        O|1|^^         123456||^^^A^^1:10\\^^^B|A||||||N
        L|1|N"""
    assert result == expected
