# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "hemohub"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&||||||||||P|1.0
    P|1|036666666|L036666666||NYME^ANO|NJF|20110513|F|||||PRESCRIBER||||||||||||LOC
    O|1|123456||^^^A|S|20130603110342|||||A
    O|2|123456||^^^B|S|20130603110342|||||A||||||||||R||||C
    O|3|123456||^^^C|S|20130603110342|||||C||||||||||||||X
    L|1|N
    """
    assert message == expected


def test_decode_result(self):
    date = datetime.datetime(2018, 2, 8, 10, 55, 32)
    worklist = self.decode(
        """
    H|\\^&|||Hemohub|||||Host||P|1.0|20180207105532
    P|1|12345699||19470801120000|M|||||12^Dr Scott|12345|CE|||SD^Without diagnosis
    C|1|L|TC^YES|G
    O|1|12345699||^^^INR|S|20180207105532
    R|1|^^^INR|Haemolysed sample|INR|0.8-1.2|HH||X|20180208105532|olga^olga
    O|2|12345699||^^^APTTE|S|20180207105532
    R|1|^^^APTTE|15.0|s||||F|20180208105532|olga^olga
    O|3|12345699||^^^APTTEr|S|20180207105532
    R|1|^^^APTTEr|0.8|Ratio||||F|20180208105532|olga^olga
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "12345699",
                "rack": None,
                "results": [
                    {
                        "code": "APTTE",
                        "date": date,
                        "flags": [],
                        "comments": [],
                        "instrument": "",
                        "value": "15.0",
                        "unit": "s",
                        "operator": "olga",
                        "validator": "olga",
                    },
                    {
                        "code": "APTTEr",
                        "date": date,
                        "flags": [],
                        "comments": [],
                        "instrument": "",
                        "operator": "olga",
                        "unit": "Ratio",
                        "validator": "olga",
                        "value": "0.8",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_with_rack(self):
    worklist = self.decode(
        """
    H|\\^&|||HEMOHUB|||||LIS||P|1.0|20180613153119
    P|1|2018061302|IPP2018061302|^^|NOM02^PRENOM02||19211004120000|M|||||Doctor 1^Doctor|EXT|EXT|||PREVISCAN^PREVISCAN||||Commentaires|||CH|||||||Service 1^Service 1|DA^
    O|1|2018061302|2018061302^01^10^|^^^TP_s|R|20180613120000|||||A|||20180613151738||Doctor 1^Doctor 1||COLA|||||||F||||Service1^Service 1|
    R|1|^^^TP_s|13.1|s||||F|20180613152530|^admin|||142
    C|1||LI-TP ReadiPlasTin^N0864563^20180801000000|
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "2018061302",
                "rack": "01-10",
                "results": [
                    {
                        "code": "TP_s",
                        "comments": ["LI-TP ReadiPlasTin"],
                        "date": datetime.datetime(2018, 6, 13, 15, 25, 30),
                        "flags": [],
                        "instrument": "142",
                        "operator": "",
                        "unit": "s",
                        "validator": "admin",
                        "value": "13.1",
                    }
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_production(self):
    self.configure(hemohub_qc_reference_concatenee=True)
    worklist = self.decode(
        """
    H|\\^&|||HEMOHUB|||||LIS||P|1.0|20180323140029
    P|1|008032301016|180322000134|^^|Z_LABOHEMOHUB^HBPM||20000201120000|M|||||MNR^Medecin Non renseigne||NR|||TNR^Traitement Non Renseigne||||||||||||||LAB^Service A|DA^|
    O|1|008032301016|008032301016|^^^HBPMXA|R|20180323120000|||||A|||20180323100819||MNR^Medecin Non renseigne||LIS COL|||||||F||||LAB^Service A|
    R|1|^^^HBPMXA|0.00|IU/mL|||Inferieur a la Limite de Linearite (5503)|F|20180323145920|^admin|||134
    C|1||LI-Anti Chromogenic Sub^N1173595^20201101000000|
    L|1|
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "008032301016",
                "rack": None,
                "results": [
                    {
                        "code": "HBPMXA",
                        "date": datetime.datetime(2018, 3, 23, 14, 59, 20),
                        "flags": ["Inferieur a la Limite de Linearite (5503)"],
                        "comments": ["LI-Anti Chromogenic Sub"],
                        "instrument": "134",
                        "value": "0.00",
                        "unit": "IU/mL",
                        "operator": "",
                        "validator": "admin",
                    }
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    self.configure(hemohub_qc_reference_concatenee=True)
    worklist = self.decode(
        """
    H|\\^&|||HEMOHUB|||||LIS||P|1.0|20180313095723
    P|1|B27765|DDHS 500 CONTROL L1||139^TOP750N^TOP 750 NDS^QC
    O|1|DDHS 500 CONTROL L1^null||^^^DDi^^QC^^139|||||||Q|||||||||||20180313105650|||F
    R|1|^^^DDi|557.0|||||F|||||139^TOP750N
    C|1||LI-DDHS 500 Control L1^B27765^20181101000000|
    C|2||LI-DDi Buffer^B27806^20181201000000|
    C|3||LI-DDi Latex^B27806^20181201000000|
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "control_lot": "DDHS 500 CONTROL L1-B27765",
                "control_name": "DDHS 500 CONTROL L1",
                "sample_id": "DDHS 500 CONTROL L1-B27765",
                "results": [
                    {
                        "code": "DDi",
                        "value": "557.0",
                        "flags": [],
                        "comments": [],
                        "unit": "",
                        "date": None,
                        "operator": "",
                        "validator": "",
                        "instrument": "139",
                    }
                ],
            }
        ],
    }
    assert worklist == expected
