# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

import pytest

import driverng.exceptions as exceptions
from driverng.tests import BaseTest

parser_class = "hprim"


def test_mapping(self):
    message = self.parse(
        """
    H|~^\\&|LOUV78141.HPR||743100323~MIRIALIS CLUSES|509, Avenue Paul Bhet~~Cluses~Haute-Savoie~74300~France|ORM|04.50.98.20.03||999999999~MCA|Clarilab Clarisys|P|H2.2~L|201206121550
P|1||110310006371|1143~1|ANO~NYME~~HUM~Mr||19470516|M||41 RUE DES ALLOBROGES~1143~SALLANCHES~Haute-Savoie~74703||04.50.91.85.14|741035067~MEDECIN&Ano&&NYME&Dr~L|||||||||||IP|~~DIALY
A|SE
OBR|1|100001668999~1000016699||CA^PHO~LABEL1^KS^KSHEM^ASPHE^CAHEM^ASPI^ASPLA^PHOHEM|A|20120612154937|201206121549|||740788617~CENTRE HEMODIALYSE DU MONT BLANC|A||||SEC&ROUGE&L|||||||||DIALYSE~SFDTM - DIALYS
A|E CLUSES|||||||||||ECD~ANO NYME&Ano&&&Mlle
OBX|1|NM|CA~CALCIUM~L|1|2.14|MMOL/L~mmol/l||L||||||201103221649
OBX|2|NM|PHO~PHOSPHORE~L|1|0.61|MMOL/L~mmol/l||D||||||201206051710
OBR|2|100001668999~1000016699||AVK0^INR^AVK1~~~CODE2~LABEL2^AVK2^AVK5^AVK3^AVK4|A|20120612154937|201206121549|||740788617~CENTRE HEMODIALYSE DU MONT BLANC|A||||CIT&BLEU&L|||||||||DIALYSE~SFDTM - DIALYSE CLUSES|||||
A|||||||ECD~ANO NYME&Ano&&&Mlle
OBX|1|CE|AVK0~Traitement :~L|1|||||||A|||201206121550
OBX|2|NM|INR~I. N. R.  .  .  .  .  .  :~L|1|1.55|||L||||||201206091623
OBX|3|CE|AVK1~Posologie :~L|1|||||||A|||201206121550
OBX|4|CE|AVK2~~L|1|||||||A|||201206121550
OBX|5|CE|AVK5~Cible et Zone thapeutique:~L|1|||||||A|||201206121550
OBX|6|CE|AVK3~~L|1|||||||A|||201206121550
L|1||1|13"""
    )
    assert message.emetteur.nom == "MIRIALIS CLUSES"
    assert message.telephone[0] == "04.50.98.20.03"
    assert message.adresse.code_postal == "74300"
    obr1 = message.patients[0].orders[0]
    obr2 = message.patients[0].orders[1]
    assert [
        ce.code1 for ce in obr1.prescription
    ] == "CA^PHO^KS^KSHEM^ASPHE^CAHEM^ASPI^ASPLA^PHOHEM".split("^")
    assert obr2.prescription[2].label2 == "LABEL2"
    obx = obr2.results[1]
    assert obx.resu[0].code1 == "1.55"
    assert obx.date == datetime.datetime.strptime("201206091623", "%Y%m%d%H%M")


def test_legacy_attributes(self):
    message = self.parse(
        """
    H|~^\\&|
    P|1
    OBR|1|||||||||||||||PRS~PRESCRIPTEUR~L~SER~SERVICE~L
    L|1"""
    )
    assert message.patients[0].orders[0].prescripteur[0].nom_medecin == "PRESCRIPTEUR"
    assert message.patients[0].orders[0].prescripteur[0].medecin.nom == "PRESCRIPTEUR"

    assert message.patients[0].orders[0].prescripteur[0].nom_service == "SERVICE"
    assert message.patients[0].orders[0].prescripteur[0].service.nom == "SERVICE"


def test_mapping_diagnostic_patient(self):
    message = self.parse(
        """
    H|~^\\&|
    P|1||110310006371|1143~1|ANO~NYME~~HUM~Mr|||||||||||||foo^bar~zaz
    OBR|1|100001668999~1000016699||CA
    L|1"""
    )
    assert message.patients[0].diag_pat == "foo"
    dp = message.patients[0].diagnostic_patient
    assert dp[0].code1 == "foo"
    assert dp[1].code1 == "bar"
    assert dp[1].label1 == "zaz"


def test_escape(self):
    message = self.parse(
        """
    H|^~\\&|||38-1^LMX||ORM|||OL2^SIEMENS||P|H2.1|20131025112139
    P|0001|9700910||009292355|ANO^NYME^^^MME^|FERON|19551115|F||42 RUE BIDON^^62170 MONTREUIL SUR MER F^||0321909684
    OBR|0001|1125249^3102525249||NAU^Sodium Urinaire^L~KU^Potassium Urinaire^L~CLU^Chlore Urinaire^L~CREAU^Créat Urinaire^L~MALBU^Microalbuminurie^L~CAUR^Ca Urinaire^L~CREAU^Créat Urinaire^L~RPROT^Protéinurie^L~RPROT^Protéinurie^L|R|201310250926|201310250933||||R||DR=\\S\\Vol=1800\\S\\Temp.=\\S\\S\\S\\S\\S\\|201310251011|11|ADELI^C.PELLE^L^9221^Médecine Cardiologie A^L|
    C|1|L|DR=\\S\\Vol=1800\\S\\Temp.=\\S\\S\\S\\S\\S\\
    OBX|1|CE|RPROT||PACJ^P.non reçu, demande annulée^L|||||||||20131024
    OBX|2|CE|RPROT||PACJ^P.non reçu, demande annulée^L|||||||||20131024
    L|1||1|8
    """
    )
    assert (
        message.patients[0].orders[0].renseignement_clinique
        == "DR=~Vol=1800~Temp.=~S~S~"
    )
    assert message.patients[0].orders[0].prescripteur[0].code_medecin == "ADELI"


_order_message = """
H|~^\\&|||SENDER||ORM|||RECEIVER||P|H2.1~L|20130715141912
P|1|036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||0123456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58|Diagnostic du SIL|MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
OBR|1|123456~78910||A|A|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
OBX|1|NM|A||42|mV||||||||||||INST
OBR|2|123456~78910||B|A|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|A|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
OBX|1|DT|B||201302131516|||||||||20110506121300||||INST
OBR|3|123456~78910||C~~~ANAL1|A|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|C|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
L|1||1|8
"""

_order_message_with_patient_comment = """
H|~^\\&|||SENDER||ORM|||RECEIVER||P|H2.1~L|20130715141912
P|1|036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||0123456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58|Diagnostic du SIL|MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
C|1||Commentaire patient no1^Commentaire patient no2
OBR|1|123456~78910||A|A|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
OBX|1|NM|A||42|mV||||||||||||INST
OBR|2|123456~78910||B|A|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|A|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
OBX|1|DT|B||201302131516|||||||||20110506121300||||INST
OBR|3|123456~78910||C~~~ANAL1|A|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|C|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
L|1||1|9
"""

_order_message_with_option_envoi_sample_id = """
H|~^\\&|||SENDER||ORM|||RECEIVER||P|H2.1~L|20130715141912
P|1|036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||0123456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58|Diagnostic du SIL|MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
OBR|1|123456~123456||A|A|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
OBX|1|NM|A||42|mV||||||||||||INST
OBR|2|123456~123456||B|A|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|A|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
OBX|1|DT|B||201302131516|||||||||20110506121300||||INST
OBR|3|123456~123456||C~~~ANAL1|A|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|C|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
L|1||1|8
"""

_order_message_nature = """
H|~^\\&|||SENDER||ORM|||RECEIVER||P|H2.1~L|20130715141912
P|1|036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||0123456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58|Diagnostic du SIL|MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
OBR|1|123456~78910||A|A|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N||||URINE&Urine&L~Aucun|PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
OBX|1|NM|A||42|mV||||||||||||INST
OBR|2|123456~78910||B|A|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|A||||URINE&Urine&L~Aucun|PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
OBX|1|DT|B||201302131516|||||||||20110506121300||||INST
OBR|3|123456~78910||C~~~ANAL1|A|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|C||||URINE&Urine&L~Aucun|PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
L|1||1|8
"""

_order_message_mobile_number_only = """
H|~^\\&|||SENDER||ORM|||RECEIVER||P|H2.1~L|20130715141912
P|1|036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||^0623456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58|Diagnostic du SIL|MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
OBR|1|123456~78910||A|A|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
OBX|1|NM|A||42|mV||||||||||||INST
OBR|2|123456~78910||B|A|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|A|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
OBX|1|DT|B||201302131516|||||||||20110506121300||||INST
OBR|3|123456~78910||C~~~ANAL1|A|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|C|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
L|1||1|8
"""


def test_encode_order(self):
    message = self.order()
    assert message == _order_message


def test_encode_order_with_patient_comment(self):
    self.configure(option_envoi_commentaire_patient=True)
    message = self.order()
    assert message == _order_message_with_patient_comment


def test_encode_order_with_option_envoi_sample_id(self):
    self.configure(option_envoi_sample_id=True)
    message = self.order()
    assert message == _order_message_with_option_envoi_sample_id


def test_encode_order_nature(self):
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0].nature = {
        "code": "URINE",
        "label": "Urine",
        "codification": "L",
        "additive": "Aucun",
    }
    message = self.result(worklist=worklist)
    assert message == _order_message_nature
    wl = self.decode(_order_message_nature)
    assert wl.samples[0].nature == worklist.samples[0].nature


def test_encode_order_poccelerator(self):
    self.configure(hprim_orm_poccelerator=True)
    message = self.order(context={"context": "result"})
    expected = """
    H|~^\\&|||SENDER||ORM|||RECEIVER||P|H2.1~L|20130715141912
    P|1|036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||0123456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58|Diagnostic du SIL|MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
    OBR|1|123456~78910||A|A|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
    OBX|1|NM|A||42|mV||||||||||||INST
    OBR|2|123456~78910||B|A|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|A|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
    OBX|1|DT|B||201302131516|||||||||||||INST
    OBR|3|123456~78910||C~~~ANAL1|A|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|C|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
    L|1||1|8
    """
    assert message == expected


def test_decode_order_service_executant(self):
    message = """
    H|~^\\&|||SENDER||ORM|||RECEIVER||P|H2.1~L|20130715141912
    P|1|036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||0123456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58|Diagnostic du SIL|MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
    OBR|1|123456~78910||A|A|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N|||||PRESCRIBER~Prescriber|0123^456|||||||SERV_EXE~SERVICE_EXECUTANT|||||||||||OPE
    L|1||1|4
    """
    worklist = self.decode(message)
    assert worklist.samples[0].services[0].code == "SERV_EXE"
    assert worklist.samples[0].services[0].name == "SERVICE_EXECUTANT"


def test_decode_order_with_rack(self):
    self.configure(option_hprim_integre_rack=True)
    message = """
    H|~^\\&|||SENDER||ORM|||RECEIVER||P|H2.1~L|20130715141912
    P|1|036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||0123456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58||MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
    OBR|1|123456~78910||A|A|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N|||||PRESCRIBER~Prescriber|0123^456|||||||SERV_EXE~SERVICE_EXECUTANT|||||||||||OPE
    OBX|1|NM|RACK||A1-001||||||P|||||auto||Xn9000
    L|1||1|4
    """
    worklist = self.decode(message)
    assert worklist.samples[0].rack == "A1-001"


def test_decode_order_with_option_identification_demandeur(self):
    self.configure(option_identification_demandeur=True)
    message = """
    H|~^\\&|||SENDER||ORM|||RECEIVER||P|H2.1~L|20130715141912
    P|1|036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||0123456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58||MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
    OBR|1|123456~78910||A|A|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N|||||PRESCRIBER~Prescriber|0123^456|dix-neuf|libelle2_demandeur|||||SERV_EXE~SERVICE_EXECUTANT|||vingt||||||||OPE
    L|1||1|4
    """
    worklist = self.decode(message)
    assert worklist.samples[0].request_info == "dix-neuf"
    assert worklist.samples[0].comments == [
        {"text": "libelle2_demandeur", "type": "sample"}
    ]


def test_decode_order_multiple_prescripteur(self):
    message = """
    H|~^\\&|||SENDER||ORM|||RECEIVER||P|H2.1~L|20130715141912
    P|1|036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||0123456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58|Diagnostic du SIL|MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
    OBR|1|99SN00656256~C1112280170||CFX_IC_CT^CFX_E_CT^CFX_N_CT^CFX_IDR_COV^CFX_IDR_GRIP^CFX_RESU^CFX_RESU_2|R|202111221458|20211122150500|||8LAC~LAURET Coraline|N|||||VITSA~DR VITAUX Sandrine^COVID~AMELI SANS PRESCRIPTION^~^|0468283722|8||||||L~Prélèvement LABORATOIRE||||||
    L|1||1|4
    """
    worklist = self.decode(message)
    assert worklist.samples[0].physicians[1].code == "VITSA"
    assert worklist.samples[0].physicians[1].last_name == "DR VITAUX Sandrine"
    assert worklist.samples[0].physicians[1].phone == "0468283722"
    assert worklist.samples[0].physicians[2].code == "COVID"
    assert worklist.samples[0].physicians[2].last_name == "AMELI SANS PRESCRIPTION"
    assert worklist.samples[0].physicians[2].phone is None


def test_decode_order_option_ignorer_service_prescripteur(self):
    message = """
    H|^~\\&||000|SENDER^Laboratoire SENDER|  |ORM|| |99-MCA^RECEIVER| |P|H2.4^L|201803011119
    P|1|03666666|||NYME^ANO||19910524|F||56 RUE LAMBDA^^LALA^^31000|9999999^INS-C^20180301|||^C|||||||||20180301
    OBR|1|123456^78910||A^A 4A|R||201803011102~201802011200||||N|||||PRESCRIBER^prescriber&firstname^R^Clinique AAA avec adresse&&31000
    L|1||1|1
    """
    worklist = self.decode(message)
    expected = {
        "context": "order",
        "receiver_id": "99-MCA",
        "sender_id": "SENDER",
        "samples": [
            {
                "address": "56 RUE LAMBDA",
                "admission_date": datetime.datetime(2018, 3, 1, 0, 0),
                "admission_end_date": None,
                "admission_status": "",
                "alternate_patient_id": "",
                "bed": "",
                "birth_row": "",
                "birthdate": datetime.datetime(1991, 5, 24, 0, 0),
                "city": "LALA",
                "collection_date": datetime.datetime(2018, 3, 1, 11, 2),
                "collection_end_date": None,
                "country": "",
                "deathdate": None,
                "diagnostic": "",
                "exe_request_id": "",
                "exe_sample_id": "",
                "first_name": "ANO",
                "height": "",
                "last_name": "NYME",
                "location": "",
                "maiden_name": "",
                "orders": [
                    {"action": "new", "code": "A", "label": "A 4A", "priority": "R"}
                ],
                "patient_id": "03666666",
                "phone": "",
                "physicians": [
                    {
                        "code": "PRESCRIBER",
                        "last_name": "prescriber",
                        "first_name": "firstname",
                        "phone": "",
                        "codification": "R",
                        "type": "prescriber",
                    }
                ],
                "physiologie": "",
                "postal_code": "31000",
                "request_date": datetime.datetime(2018, 3, 1, 11, 2),
                "request_id": "78910",
                "room": "",
                "sample_id": "123456",
                "sex": "F",
                "title": "",
                "transcriptionist": "",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_order_with_multiples_results(self):
    worklist = self.decode(
        """
    H|~^\\&|||LABOTEST||ORM|||MCA||P|H2.2~L|2013012416522
    P|1|123456789|||Nom~Prenom||19000101|M||||
    OBR|1|12345678910~4242||A|R||201402270645||||N
    OBX|1|CE|A|1|foo
    OBX|1|CE|A|2|bar
    OBX|1|CE|A|1|zap|||||||||201401071000
    OBX|1|CE|A|2|zou|||||||||201401071000
    OBX|1|CE|A|1|foo|||||||||201401071132
    OBX|1|CE|A|2|zaz|||||||||201401071132
    L|1||1|6
    """
    )
    expected = {
        "context": "order",
        "receiver_id": "MCA",
        "sender_id": "LABOTEST",
        "samples": [
            {
                "address": "",
                "admission_date": None,
                "admission_end_date": None,
                "admission_status": "",
                "alternate_patient_id": "",
                "bed": "",
                "birth_row": "",
                "birthdate": datetime.datetime(1900, 1, 1, 0, 0),
                "city": "",
                "collection_date": datetime.datetime(2014, 2, 27, 6, 45),
                "collection_end_date": None,
                "country": "",
                "deathdate": None,
                "diagnostic": "",
                "exe_request_id": "",
                "exe_sample_id": "",
                "first_name": "Prenom",
                "height": "",
                "last_name": "Nom",
                "location": "",
                "maiden_name": "",
                "orders": [{"action": "new", "code": "A", "priority": "R"}],
                "patient_id": "123456789",
                "phone": "",
                "physiologie": "",
                "postal_code": "",
                "request_date": datetime.datetime(2014, 2, 27, 6, 45),
                "request_id": "4242",
                "results": [
                    {
                        "alt_code": "",
                        "alt_codification": "",
                        "alt_label": "",
                        "alt_unit": "",
                        "alt_unit_codification": "",
                        "alt_unit_label": "",
                        "anterior": False,
                        "code": "A",
                        "codification": "",
                        "date": None,
                        "flags": [],
                        "instrument": "",
                        "label": "",
                        "sub_code": "1",
                        "type": "CE",
                        "unit": "",
                        "unit_codification": "",
                        "unit_label": "",
                        "value": "foo",
                        "value_label": "",
                    },
                    {
                        "alt_code": "",
                        "alt_codification": "",
                        "alt_label": "",
                        "alt_unit": "",
                        "alt_unit_codification": "",
                        "alt_unit_label": "",
                        "anterior": False,
                        "code": "A",
                        "codification": "",
                        "date": None,
                        "flags": [],
                        "instrument": "",
                        "label": "",
                        "sub_code": "2",
                        "type": "CE",
                        "unit": "",
                        "unit_codification": "",
                        "unit_label": "",
                        "value": "bar",
                        "value_label": "",
                    },
                    {
                        "alt_code": "",
                        "alt_codification": "",
                        "alt_label": "",
                        "alt_unit": "",
                        "alt_unit_codification": "",
                        "alt_unit_label": "",
                        "anterior": True,
                        "code": "A",
                        "codification": "",
                        "date": datetime.datetime(2014, 1, 7, 10, 0),
                        "flags": [],
                        "instrument": "",
                        "label": "",
                        "request_date": None,
                        "request_id": "",
                        "sub_code": "1",
                        "type": "CE",
                        "unit": "",
                        "unit_codification": "",
                        "unit_label": "",
                        "value": "zap",
                        "value_label": "",
                    },
                    {
                        "alt_code": "",
                        "alt_codification": "",
                        "alt_label": "",
                        "alt_unit": "",
                        "alt_unit_codification": "",
                        "alt_unit_label": "",
                        "anterior": True,
                        "code": "A",
                        "codification": "",
                        "date": datetime.datetime(2014, 1, 7, 10, 0),
                        "flags": [],
                        "instrument": "",
                        "label": "",
                        "request_date": None,
                        "request_id": "",
                        "sub_code": "2",
                        "type": "CE",
                        "unit": "",
                        "unit_codification": "",
                        "unit_label": "",
                        "value": "zou",
                        "value_label": "",
                    },
                    {
                        "alt_code": "",
                        "alt_codification": "",
                        "alt_label": "",
                        "alt_unit": "",
                        "alt_unit_codification": "",
                        "alt_unit_label": "",
                        "anterior": True,
                        "code": "A",
                        "codification": "",
                        "date": datetime.datetime(2014, 1, 7, 11, 32),
                        "flags": [],
                        "instrument": "",
                        "label": "",
                        "request_date": None,
                        "request_id": "",
                        "sub_code": "1",
                        "type": "CE",
                        "unit": "",
                        "unit_codification": "",
                        "unit_label": "",
                        "value": "foo",
                        "value_label": "",
                    },
                    {
                        "alt_code": "",
                        "alt_codification": "",
                        "alt_label": "",
                        "alt_unit": "",
                        "alt_unit_codification": "",
                        "alt_unit_label": "",
                        "anterior": True,
                        "code": "A",
                        "codification": "",
                        "date": datetime.datetime(2014, 1, 7, 11, 32),
                        "flags": [],
                        "instrument": "",
                        "label": "",
                        "request_date": None,
                        "request_id": "",
                        "sub_code": "2",
                        "type": "CE",
                        "unit": "",
                        "unit_codification": "",
                        "unit_label": "",
                        "value": "zaz",
                        "value_label": "",
                    },
                ],
                "room": "",
                "sample_id": "12345678910",
                "sex": "M",
                "title": "",
                "transcriptionist": "",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_order(self):
    worklist = self.decode(_order_message)
    expected = {
        "context": "order",
        "receiver_id": "RECEIVER",
        "sender_id": "SENDER",
        "samples": [
            {
                "patient_id": "036666666",
                "alternate_patient_id": "ADM_CLI",
                "request_date": datetime.datetime(2013, 6, 3, 11, 3, 42),
                "request_id": "78910",
                "bed": "42",
                "birth_row": "",
                "birthdate": datetime.datetime(2011, 5, 13, 0, 0),
                "collection_date": datetime.datetime(2013, 5, 13, 11, 42, 42),
                "collection_end_date": datetime.datetime(2013, 5, 13, 12, 42, 42),
                "exe_request_id": "",
                "exe_sample_id": "",
                "first_name": "ANO",
                "last_name": "NYME",
                "title": "Mme",
                "height": "1.70",
                "weight": "58.0",
                "phone": "0123456789",
                "postal_code": "31000",
                "country": "FR",
                "city": "Toulouse",
                "address": "45 rue lambda~lala",
                "location": "LOC",
                "maiden_name": "NJF",
                "deathdate": None,
                "diagnostic": "Diagnostic du SIL",
                "admission_date": datetime.datetime(2013, 5, 11),
                "admission_end_date": datetime.datetime(2013, 5, 14),
                "admission_status": "",
                "medications": ["MEDOC1", "MEDOC2"],
                "physiologie": "",
                "services": [{"code": "SER", "name": "SERVICE"}],
                "physicians": [
                    {
                        "code": "ATTENDING",
                        "alias": "",
                        "type": "attending",
                        "first_name": "Joseph-Louis",
                        "last_name": "LAGRANGE",
                        "codification": "L",
                        "title": "",
                    },
                    {
                        "code": "PRESCRIBER",
                        "type": "prescriber",
                        "codification": "",
                        "last_name": "Prescriber",
                        "first_name": "",
                        "phone": "0123;456",
                    },
                    {
                        "code": "SAMPLER",
                        "alias": "",
                        "type": "sampler",
                        "last_name": "Sampler",
                        "first_name": "",
                        "codification": "L",
                        "title": "",
                    },
                ],
                "transcriptionist": "OPE",
                "orders": [
                    {"action": "new", "code": "A", "priority": "A"},
                    {"action": "add", "code": "B", "priority": "A"},
                    {
                        "action": "del",
                        "code": "C",
                        "priority": "A",
                        "alt_code": "ANAL1",
                    },
                ],
                "results": [
                    {
                        "type": "NM",
                        "anterior": False,
                        "code": "A",
                        "date": None,
                        "value": "42",
                        "value_label": "",
                        "label": "",
                        "codification": "",
                        "alt_code": "",
                        "alt_label": "",
                        "alt_codification": "",
                        "flags": [],
                        "sub_code": "",
                        "unit": "mV",
                        "unit_label": "",
                        "unit_codification": "",
                        "alt_unit": "",
                        "alt_unit_label": "",
                        "alt_unit_codification": "",
                        "instrument": "",
                    },
                    {
                        "type": "DT",
                        "anterior": True,
                        "request_id": "",
                        "request_date": None,
                        "code": "B",
                        "date": datetime.datetime(2011, 5, 6, 12, 13),
                        "value": "201302131516",
                        "value_label": "",
                        "label": "",
                        "codification": "",
                        "alt_code": "",
                        "alt_label": "",
                        "alt_codification": "",
                        "flags": [],
                        "sub_code": "",
                        "unit": "",
                        "unit_label": "",
                        "unit_codification": "",
                        "alt_unit": "",
                        "alt_unit_label": "",
                        "alt_unit_codification": "",
                        "instrument": "",
                    },
                ],
                "room": "237",
                "sample_id": "123456",
                "sex": "F",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


_result_message = """
H|~^\\&|||SENDER||ORU|||RECEIVER||P|H2.1~L|20130715141912
P|1|L036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||0123456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58|Diagnostic du SIL|MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
OBR|1|123456~78910|123456~78910|A^B^C~~~ANAL1|R|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
OBX|1|NM|A|SA|42|mV||||||||20110506121400||VAL||INST
OBX|2|CE|B|SB|NORESULT|mV||HH^>||||||20110506121400||VAL||INST
OBX|3|NM|C~~~ANAL1|SC|42|mV||||||||20110506121400||VAL||INST
C|1|L|Abnormal
C|2|L|EE
L|1||1|9
"""


def test_encode_result(self):
    message = self.result()
    assert message == _result_message


def test_encode_result_clarilab(self):
    worklist = copy.deepcopy(self.result_worklist)
    worklist.samples[0].results = [
        {
            "code": "ANTI",
            "type": "CE",
            "value": [
                {
                    "value": "BACT",
                    "label": "POSITIVE",
                    "codification": "L",
                    "antibiogram": [
                        {"antibiotic": "ABMB", "mic": "#", "sensitivity": "BMR"},
                        {"antibiotic": "A1", "mic": "<=1", "sensitivity": "S"},
                        {"antibiotic": "A2", "mic": ">2", "sensitivity": "R"},
                    ],
                }
            ],
        },
        {
            "code": "CPLX",
            "type": "CE",
            "value": [
                {
                    "value": "CPLX",
                    "label": "Cplx",
                    "codification": "L",
                    "multiple_clarilab": [
                        "AB",
                        [2, "C", "D", "EF"],
                        "G",
                        ["H", "IJ"],
                        1,
                    ],
                }
            ],
        },
    ]
    message = self.result(worklist=worklist)
    expected = """
    H|~^\\&|||SENDER||ORU|||RECEIVER||P|H2.1~L|20130715141912
    P|1|L036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||0123456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58|Diagnostic du SIL|MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
    OBR|1|123456~78910|123456~78910|ANTI^CPLX|R|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
    OBX|1|CE|ANTI||BACT~POSITIVE~L/ABMB\\#\\BMR^A1\\<=1\\S^A2\\>2\\R
    OBX|2|CE|CPLX||CPLX~Cplx~L/AB^2\\C\\D\\EF^G^H\\IJ^1
    L|1||1|6"""
    assert message == expected


def test_encode_result_multi_echantillon(self):
    worklist = copy.deepcopy(self.result_worklist)
    worklist.samples.append(copy.deepcopy(worklist.samples[0]))
    worklist.samples[0].sample_id = "12345678911"
    worklist.samples[0].results = [
        {
            "status": "P",
            "code": "T1",
            "lots": [],
            "value": [{"codification": "", "value": "11", "label": ""}],
            "instrument": "Advia2400_1",
            "validator": "auto",
            "comments": [],
            "type": "NM",
            "unit": None,
        }
    ]
    worklist.samples[1].sample_id = "12345678912"
    worklist.samples[1].results = [
        {
            "status": "P",
            "code": "T2",
            "lots": [],
            "value": [{"codification": "", "value": "12", "label": ""}],
            "instrument": "Advia2400_1",
            "validator": "auto",
            "comments": [],
            "type": "NM",
            "unit": None,
        }
    ]
    message = self.result(worklist=worklist)
    expected = """
    H|~^\\&|||SENDER||ORU|||RECEIVER||P|H2.1~L|20130715141912
    P|1|L036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||0123456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58|Diagnostic du SIL|MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
    OBR|1|12345678911~78910|12345678911~78910|T1|R|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
    OBX|1|NM|T1||11||||||P|||||auto||Advia2400_1
    OBR|2|12345678912~78910|12345678912~78910|T2|R|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
    OBX|1|NM|T2||12||||||P|||||auto||Advia2400_1
    L|1||1|7"""
    assert message == expected


def test_decode_result(self):
    worklist = self.decode(_result_message)
    expected = {
        "context": "result",
        "receiver_id": "RECEIVER",
        "sender_id": "SENDER",
        "samples": [
            {
                "bed": "42",
                "birth_row": "",
                "birthdate": datetime.datetime(2011, 5, 13, 0, 0),
                "collection_date": datetime.datetime(2013, 5, 13, 11, 42, 42),
                "collection_end_date": datetime.datetime(2013, 5, 13, 12, 42, 42),
                "first_name": "ANO",
                "last_name": "NYME",
                "title": "Mme",
                "height": "1.70",
                "weight": "58.0",
                "phone": "0123456789",
                "postal_code": "31000",
                "country": "FR",
                "city": "Toulouse",
                "address": "45 rue lambda~lala",
                "location": "LOC",
                "maiden_name": "NJF",
                "deathdate": None,
                "diagnostic": "Diagnostic du SIL",
                "admission_date": datetime.datetime(2013, 5, 11),
                "admission_end_date": datetime.datetime(2013, 5, 14),
                "admission_status": "",
                "medications": ["MEDOC1", "MEDOC2"],
                "physiologie": "",
                "services": [{"code": "SER", "name": "SERVICE"}],
                "physicians": [
                    {
                        "alias": "",
                        "code": "ATTENDING",
                        "first_name": "Joseph-Louis",
                        "last_name": "LAGRANGE",
                        "type": "attending",
                        "codification": "L",
                        "title": "",
                    },
                    {
                        "code": "PRESCRIBER",
                        "last_name": "Prescriber",
                        "first_name": "",
                        "phone": "0123;456",
                        "type": "prescriber",
                        "codification": "",
                    },
                    {
                        "code": "SAMPLER",
                        "alias": "",
                        "type": "sampler",
                        "last_name": "Sampler",
                        "first_name": "",
                        "codification": "L",
                        "title": "",
                    },
                ],
                "transcriptionist": "OPE",
                "patient_id": "L036666666",
                "alternate_patient_id": "ADM_CLI",
                "request_date": datetime.datetime(2013, 6, 3, 11, 3, 42),
                "request_id": "78910",
                "exe_request_id": "78910",
                "exe_sample_id": "123456",
                "results": [
                    {
                        "code": "A",
                        "sub_code": "SA",
                        "type": "NM",
                        "anterior": False,
                        "date": datetime.datetime(2011, 5, 6, 12, 14),
                        "value": "42",
                        "value_label": "",
                        "label": "",
                        "codification": "",
                        "alt_code": "",
                        "alt_label": "",
                        "alt_codification": "",
                        "flags": [],
                        "unit": "mV",
                        "unit_label": "",
                        "unit_codification": "",
                        "alt_unit": "",
                        "alt_unit_label": "",
                        "alt_unit_codification": "",
                        "validator": "VAL",
                        "instrument": "",
                    },
                    {
                        "code": "B",
                        "sub_code": "SB",
                        "type": "CE",
                        "anterior": False,
                        "date": datetime.datetime(2011, 5, 6, 12, 14),
                        "value": "NORESULT",
                        "value_label": "",
                        "label": "",
                        "codification": "",
                        "alt_code": "",
                        "alt_label": "",
                        "alt_codification": "",
                        "flags": [],
                        "unit": "mV",
                        "unit_label": "",
                        "unit_codification": "",
                        "alt_unit": "",
                        "alt_unit_label": "",
                        "alt_unit_codification": "",
                        "validator": "VAL",
                        "instrument": "",
                    },
                    {
                        "code": "C",
                        "sub_code": "SC",
                        "type": "NM",
                        "anterior": False,
                        "date": datetime.datetime(2011, 5, 6, 12, 14),
                        "value": "42",
                        "value_label": "",
                        "label": "",
                        "codification": "",
                        "alt_code": "ANAL1",
                        "alt_label": "",
                        "alt_codification": "",
                        "flags": [],
                        "unit": "mV",
                        "unit_label": "",
                        "unit_codification": "",
                        "alt_unit": "",
                        "alt_unit_label": "",
                        "alt_unit_codification": "",
                        "validator": "VAL",
                        "instrument": "",
                    },
                ],
                "room": "237",
                "sample_id": "123456",
                "sex": "F",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_acceuil(self):
    message = """
    H|~^\\&|DA13P280.HPR||IASAXBIO~INOVIE AXBIO||ORM|||MCA~INOVIE PLT LABOSUD PROVENCE|1.2.250.1.297.1.1.3021674~|P|H2.4~L|202208301744
    P|1|C2010500679|||TEST~HPRIM||19800101|F||||||~C|||||||||||~~GRATU
    OBR|2|A132420134PT~20830E0134||YTACRO~TACROLIMUS~L|R||202208301751||||N|||||640000006:test informatique~test informatique~A~~~~test adresse 1&test adresse 2&BAYONNE&&64000
    OBR|3|A132420134PY~20830E0134||YVITB6~VITAMINE B6~L|R||202208301751||||N|||||640000006:test informatique~test informatique~A~~~~test adresse 1&test adresse 2&BAYONNE&&64000
    OBX|1|TX|YTACSA~Posologie~L||1/2||||||||||INOLP~999~YTACR~12~3~B
    OBX|2|CE|YTACLS~Spécialité~L||TACAD||||||||||INOLP~999~YTACR~12~2~
    L|1||1|6"""
    worklist = self.decode(message)
    assert worklist.samples[1].orders[1].accueil
    assert worklist.samples[1].orders[2].accueil


def test_result_evm(self):
    message = """
    H|~^\\&|||HISTONE3~PLATEAU TECHNIQUE LABO||ORU|||JOUE~VALBIOLAB JOUE LES TOURS (CLARISYS)||P|H2.1~L|201704051326
    P|1|47930|||WOLFF~Florence|WOLFF|19760626|F|||
    OBR|1|13095046E1~13095046||RAI|R||20170405084100||||N||||||||||||||F|||||||caro
    OBX|1|CE|RAI||NEG||||||F||VISIO2-J60002261|20170405132500|VISIO2-J60002261|caro
    OBX|2|FIC|RAI|1|EVM~2017-04/CR622-13095046E1-170405002601.pdf~ACROBAT READER||||||||VISIO2-J60002261|20170405132444|VISIO2-J60002261|caro
    L|1||1|6"""
    worklist = self.decode(message)
    expected = {
        "context": "result",
        "receiver_id": "JOUE",
        "samples": [
            {
                "address": "",
                "admission_date": None,
                "admission_end_date": None,
                "admission_status": "",
                "alternate_patient_id": "",
                "bed": "",
                "birth_row": "",
                "birthdate": datetime.datetime(1976, 6, 26, 0, 0),
                "city": "",
                "collection_date": datetime.datetime(2017, 4, 5, 8, 41),
                "collection_end_date": None,
                "country": "",
                "deathdate": None,
                "diagnostic": "",
                "exe_request_id": "",
                "exe_sample_id": "",
                "first_name": "Florence",
                "height": "",
                "last_name": "WOLFF",
                "location": "",
                "maiden_name": "WOLFF",
                "patient_id": "47930",
                "physiologie": "",
                "phone": "",
                "postal_code": "",
                "request_date": datetime.datetime(2017, 4, 5, 8, 41),
                "request_id": "13095046",
                "results": [
                    {
                        "chapter": {"code": "VISIO2-J60002261"},
                        "anterior": False,
                        "code": "RAI",
                        "label": "",
                        "codification": "",
                        "alt_code": "",
                        "alt_label": "",
                        "alt_codification": "",
                        "date": datetime.datetime(2017, 4, 5, 13, 25),
                        "flags": [],
                        "instrument": "VISIO2-J60002261",
                        "sub_code": "",
                        "type": "CE",
                        "unit": "",
                        "unit_label": "",
                        "unit_codification": "",
                        "alt_unit": "",
                        "alt_unit_label": "",
                        "alt_unit_codification": "",
                        "validator": "caro",
                        "value": "NEG",
                        "value_label": "",
                    },
                    {
                        "chapter": {"code": "VISIO2-J60002261"},
                        "anterior": False,
                        "code": "RAI_1",
                        "label": "",
                        "codification": "",
                        "alt_code": "",
                        "alt_label": "",
                        "alt_codification": "",
                        "date": datetime.datetime(2017, 4, 5, 13, 24, 44),
                        "flags": [],
                        "instrument": "VISIO2-J60002261",
                        "sub_code": "1",
                        "type": "FIC",
                        "unit": "",
                        "unit_label": "",
                        "unit_codification": "",
                        "alt_unit": "",
                        "alt_unit_label": "",
                        "alt_unit_codification": "",
                        "validator": "caro",
                        "value": "2017-04/CR622-13095046E1-170405002601.pdf",
                        "value_label": "",
                    },
                ],
                "room": "",
                "sample_id": "13095046E1",
                "sex": "F",
                "title": "",
                "transcriptionist": "",
                "type": "patient",
            }
        ],
        "sender_id": "HISTONE3",
    }
    assert worklist == expected


def test_result_fic(self):
    message = """
    H|~^\\&|||||ORU|||MCA||P|H2.1~L|201410011728
    P|1|03091683380|B4100108372||Z_DAMIEN~Emma||19801112|U|||||||||||||||||~~ARCM
    OBR|1|121245566~121245566|~41001q1072|T211|R||201410011448||||N|||201410011451|||||||||||F
    OBX|24|FIC|TEXTE~~L||HEXALIS~MCASR01.j1.pdf~PDF
    L|1"""
    worklist = self.decode(message)
    expected = {
        "context": "result",
        "receiver_id": "MCA",
        "sender_id": "",
        "samples": [
            {
                "sample_id": "121245566",
                "type": "patient",
                "room": "",
                "sex": "U",
                "admission_date": None,
                "admission_end_date": None,
                "admission_status": "",
                "bed": "",
                "birth_row": "",
                "birthdate": datetime.datetime(1980, 11, 12, 0, 0),
                "collection_date": datetime.datetime(2014, 10, 1, 14, 48),
                "collection_end_date": None,
                "deathdate": None,
                "diagnostic": "",
                "exe_request_id": "41001q1072",
                "exe_sample_id": "",
                "first_name": "Emma",
                "last_name": "Z_DAMIEN",
                "title": "",
                "height": "",
                "phone": "",
                "address": "",
                "postal_code": "",
                "city": "",
                "country": "",
                "transcriptionist": "",
                "location": "ARCM",
                "maiden_name": "",
                "patient_id": "03091683380",
                "alternate_patient_id": "",
                "request_date": datetime.datetime(2014, 10, 1, 14, 48),
                "request_id": "121245566",
                "physiologie": "",
                "results": [
                    {
                        "type": "FIC",
                        "anterior": False,
                        "code": "TEXTE",
                        "label": "",
                        "codification": "L",
                        "alt_code": "",
                        "alt_label": "",
                        "alt_codification": "",
                        "date": None,
                        "sub_code": "",
                        "value": "MCASR01.j1.pdf",
                        "value_label": "",
                        "flags": [],
                        "unit": "",
                        "unit_label": "",
                        "unit_codification": "",
                        "alt_unit": "",
                        "alt_unit_label": "",
                        "alt_unit_codification": "",
                        "instrument": "",
                    }
                ],
            }
        ],
    }
    assert worklist == expected


def test_traceback(self):
    message = """
    H|~^\\&|||||ORU|||MCA||P|H2.1~L|NOT_A~DATE
    L|1"""
    with pytest.raises(exceptions.ParsingError):
        self.decode(message)


def test_demande_anterieure_clarilab(self):
    worklist = self.decode(
        """
    H|~^\\&|||PRE1||ORM|||99999997||P|H2.2~L|201412030810
    P|1||PID
    OBR|1|SID~RID||VPMS||20141203080909|201412030809||||A
    OBX|1|NM|VPMS~VPM~L|1|10|FL~Fl~L~MG~Mg~L||||||||201411101035||||||110161080~20141110080756
    L|1|1"""
    )
    expected = {
        "context": "order",
        "sender_id": "PRE1",
        "receiver_id": "99999997",
        "samples": [
            {
                "address": "",
                "admission_date": None,
                "admission_end_date": None,
                "admission_status": "",
                "alternate_patient_id": "",
                "bed": "",
                "birth_row": "",
                "birthdate": None,
                "city": "",
                "collection_date": datetime.datetime(2014, 12, 3, 8, 9),
                "collection_end_date": None,
                "country": "",
                "deathdate": None,
                "physiologie": "",
                "diagnostic": "",
                "exe_request_id": "",
                "exe_sample_id": "",
                "first_name": "",
                "height": "",
                "last_name": "",
                "location": "",
                "maiden_name": "",
                "orders": [{"action": "add", "code": "VPMS", "priority": "R"}],
                "patient_id": "PID",
                "phone": "",
                "postal_code": "",
                "request_date": datetime.datetime(2014, 12, 3, 8, 9, 9),
                "request_id": "RID",
                "results": [
                    {
                        "anterior": True,
                        "type": "NM",
                        "code": "VPMS",
                        "label": "VPM",
                        "codification": "L",
                        "alt_code": "",
                        "alt_label": "",
                        "alt_codification": "",
                        "date": datetime.datetime(2014, 11, 10, 10, 35),
                        "request_date": datetime.datetime(2014, 11, 10, 8, 7, 56),
                        "request_id": "110161080",
                        "sub_code": "1",
                        "unit": "FL",
                        "unit_label": "Fl",
                        "unit_codification": "L",
                        "alt_unit": "MG",
                        "alt_unit_label": "Mg",
                        "alt_unit_codification": "L",
                        "value": "10",
                        "value_label": "",
                        "flags": [],
                        "instrument": "",
                    }
                ],
                "room": "",
                "sample_id": "SID",
                "sex": "U",
                "title": "",
                "transcriptionist": "",
                "type": "patient",
            }
        ],
    }

    assert worklist == expected


def test_decode_a_telephoner(self):
    worklist = self.decode(
        """
    H|~^\\&|||MCA||ORU|||MCA||P|H2.1~L|20150908115418
    P|1|PID||||||M||||~T
    OBR|1|SID
    L|1"""
    )
    assert worklist.samples[0].phone == ""


def test_action_synergie_hprim(self):
    worklist = self.decode(
        """
    H|^~\\&|||||ORM|||||P|H2.1|20150924094618
    P|0001|002258960||107023565|ANO^Nyme^^^MME^||19351231|F
    OBR|0001|05178377^5092478377||IGGSG^IgG Sérum^L|R|201509240945|201509240845||||C^T|||201509240945
    L|1||1|4"""
    )
    assert worklist.samples[0].orders[0].action == "del"


def test_decode_mobile_number_only(self):
    worklist = self.decode(_order_message_mobile_number_only)
    assert worklist.samples[0].phone == ";0623456789"


class TestLegacyHprim(BaseTest):
    MAPPING_CLASS = parser_class

    def test_legacy_order(self):
        self.assertOrderMessageEqual(
            self.WORKLIST_NEW_ADD_DEL,
            """
        H|~^\\&|||MCA||ORM|||||P|H2.1~L|20130715141912
        P|1|036666666|||NYME~ANO|NJF|20110513|F||45 rue lambda~~Toulouse~~31000||||||||Diagnostic du SIL|||||20130511||~~LOC
        OBR|1|123456~123456||A|A|20130603110342|20130513114242||||N|||||PRS~LAGRANGE~~SER
        OBX|1||A||42
        OBR|2|123456~123456||B|A|20130603110342|20130513114242||||A|||||PRS~LAGRANGE~~SER
        OBX|1||B||201302131516|||||||||20110506121300
        OBR|3|123456~123456||C|A|20130603110342|20130513114242||||C|||||PRS~LAGRANGE~~SER
        L|1||1|8
        """,
        )


def test_segment_a_en_fin(self):
    """Fichier sans segment L."""
    worklist = self.decode(
        """
    H|~^\\&|ALPES00218.HPR||CLARISYS1~LBM DES ALPES|2 rue de CHAMONIX~~Annemasse~~74000|ORM|09.72.11.43.60||MCA4~MCA|Clarilab Clarisys|P|H2.2~L|201606171330
    P|1||1310150006||MAUREL~Laure~~HUM~Mme||19821015|F||3Chemin du bois de Saget~~Saint jean~~31240|||FANABE~FANAPO&Bernard&&FANABE~L|||||||||||IP
    C|1||Patiente difficile à prélever
    OBR|1|311015000804~3110150008||BLAST^PROM^MYELO^META^PLAQ^MBLAS^PBLAS^LYMPLA^LYMATY^LYHBL^PLAQCOML^GR^VPMS^CELATY^ERY^PROMO^TRICH^HB^HT^PLAQCPF^VGM^TCMH^CCMH^IDR^GB^NEU^COMPF^NEUG^EOS^LBLAS^EOSG^BASO
    A|^BASOG^LYM^COMLIB^LYMG^MON^MONG^CAENT||20131015103617|201310151036|||ROUSMA~ROUSSEAU|A||||EDTA04&EDTA&L|FANABE~FANAPO&Bernard&&FANABE~L|05.61.50.10.50||||||||||||||||||damien~BASTIT&Damien&&&Mr"""
    )
    assert worklist.samples[0].orders[0].action == "add"


def test_encode_result_operator(self):
    self.configure(option_envoi_technicien=True)
    worklist = copy.deepcopy(self.result_worklist)
    worklist.samples[0].sample_id = "12345678911"
    worklist.samples[0].results = [
        {
            "status": "P",
            "code": "T1",
            "lots": [],
            "value": [{"codification": "", "value": "11", "label": ""}],
            "instrument": "Advia2400_1",
            "validator": "auto",
            "operator": "OPE",
            "comments": [],
            "type": "NM",
            "unit": None,
        }
    ]
    message = self.result(worklist=worklist)
    expected = """
    H|~^\\&|||SENDER||ORU|||RECEIVER||P|H2.1~L|20130715141912
    P|1|L036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||0123456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58|Diagnostic du SIL|MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
    OBR|1|12345678911~78910|12345678911~78910|T1|R|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
    OBX|1|NM|T1||11||||||P|||||OPE||Advia2400_1
    L|1||1|5"""
    assert message == expected
    worklist.samples[0].results[0].operator = ""
    worklist.samples[0].results[0].validator = ""
    message = self.result(worklist=worklist)
    expected = """
    H|~^\\&|||SENDER||ORU|||RECEIVER||P|H2.1~L|20130715141912
    P|1|L036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||0123456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58|Diagnostic du SIL|MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
    OBR|1|12345678911~78910|12345678911~78910|T1|R|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
    OBX|1|NM|T1||11||||||P|||||||Advia2400_1
    L|1||1|5"""
    assert message == expected


def test_encode_result_operator_and_validator_auto(self):
    self.configure(option_envoi_technicien=True)
    worklist = copy.deepcopy(self.result_worklist)
    worklist.samples[0].sample_id = "12345678911"
    worklist.samples[0].results = [
        {
            "status": "P",
            "code": "T1",
            "lots": [],
            "value": [{"codification": "", "value": "11", "label": ""}],
            "instrument": "Advia2400_1",
            "validator": None,
            "operator": "OPE",
            "comments": [],
            "type": "NM",
            "unit": None,
        }
    ]
    message = self.result(worklist=worklist)
    expected = """
    H|~^\\&|||SENDER||ORU|||RECEIVER||P|H2.1~L|20130715141912
    P|1|L036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||0123456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58|Diagnostic du SIL|MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
    OBR|1|12345678911~78910|12345678911~78910|T1|R|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
    OBX|1|NM|T1||11||||||P|||||OPE||Advia2400_1
    L|1||1|5"""
    assert message == expected
    self.configure(option_envoi_technicien=False, hprim_send_validator_auto=True)
    worklist.samples[0].results[0].operator = None
    message = self.result(worklist=worklist)
    expected = """
    H|~^\\&|||SENDER||ORU|||RECEIVER||P|H2.1~L|20130715141912
    P|1|L036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||0123456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58|Diagnostic du SIL|MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
    OBR|1|12345678911~78910|12345678911~78910|T1|R|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
    OBX|1|NM|T1||11||||||P|||||auto||Advia2400_1
    L|1||1|5"""
    assert message == expected


def test_repetiteur_tx(self):
    message = """H|~^\\&|||SENDER||ORU|||RECEIVER||P|H2.1~L|20130715141912
    P|1|L036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||0123456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58|Diagnostic du SIL|MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
    OBR|1|12345678911~78910|12345678911~78910|T1|R|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
    OBX|1|TX|T1||foo^bar^42||||||P|||||OPE||Advia2400_1
    L|1||1|5"""
    worklist = self.decode(message)
    assert worklist.samples[0].results[0].value == "foo\nbar\n42"


def test_repetiteur(self):
    message = """H|~^\\&|||SENDER||ORU|||RECEIVER||P|H2.1~L|20130715141912
    P|1|L036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||0123456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58|Diagnostic du SIL|MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
    OBR|1|12345678911~78910|12345678911~78910|T1|R|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
    OBX|1|CE|T1||foo^bar^42||||||P|||||OPE||Advia2400_1
    L|1||1|5"""
    worklist = self.decode(message)
    assert worklist.samples[0].results[0].value == "foo;bar;42"


_result_message_with_option_bioserveur = """
H|~^\\&|||SENDER||ORU|||RECEIVER||P|H2.1~L|20130715141912
P|1|L036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||0123456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58|Diagnostic du SIL|MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
OBR|1|~78910|~78910|A^B^C~~~ANAL1|R|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
OBX|1|NM|A|SA|42|mV||||||||20110506121400|~~A|VAL||INST
OBX|2|CE|B|SB|NORESULT|mV||HH^>||||||20110506121400|~~B|VAL||INST
OBX|3|NM|C~~~ANAL1|SC|42|mV||||||||20110506121400|~~C|VAL||INST
C|1|L|Abnormal
C|2|L|EE
L|1||1|9
"""


def test_encode_result_with_option_bioserveur(self):
    self.configure(option_bioserveur=True)
    message = self.result()
    assert message == _result_message_with_option_bioserveur


_result_message_normality = """
H|~^\\&|||SENDER||ORU|||RECEIVER||P|H2.1~L|20130715141912
P|1|L036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||0123456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58|Diagnostic du SIL|MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
OBR|1|123456~78910|123456~78910|A|R|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
OBX|1|NM|A|SA|42|mV|1.2-2.0|||||||20110506121400||VAL||INST
L|1||1|5
"""


def test_encode_result_normality(self):
    self.configure()
    worklist = copy.deepcopy(self.result_worklist)
    s = worklist.samples[0]
    s.results = [s.results[0]]
    s.results[0].normality = "1.2-2.0"
    message = self.encode(worklist=worklist)
    assert message == _result_message_normality
    wl = self.decode(message.val)
    assert wl.samples[0].results[0].normality == "1.2-2.0"


_result_message_secteurtech = """
H|~^\\&|||SENDER||ORU|||RECEIVER||P|H2.1~L|20130715141912
P|1|L036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||0123456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58|Diagnostic du SIL|MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
OBR|1|123456~78910|123456~78910|A|R|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
OBX|1|NM|A|SA|42|mV||||||||20110506121400|CHAP1~1~ANO~2~3|VAL||INST
L|1||1|5
"""


def test_encode_result_secteur_tech(self):
    self.configure()
    worklist = copy.deepcopy(self.result_worklist)
    s = worklist.samples[0]
    r = s.results[0]
    r.chapter.code = "CHAP1"
    r.chapter.position = "1"
    r.battery.code = "ANO"
    r.battery.chapter_position = "2"
    r.battery.position = "3"
    s.results = [r]
    message = self.encode(worklist=worklist)
    assert message == _result_message_secteurtech
    wl = self.decode(message.val)
    assert wl.samples[0].results[0].chapter == r.chapter
    assert wl.samples[0].results[0].battery == r.battery


def test_encode_result_validator_code_and_name(self):
    self.configure(hprim_send_validator_code_and_name=True)
    worklist = copy.deepcopy(self.result_worklist)
    worklist.samples[0].sample_id = "12345678911"
    worklist.samples[0].results = [
        {
            "status": "P",
            "code": "T1",
            "lots": [],
            "value": [{"codification": "", "value": "11", "label": ""}],
            "instrument": "Advia2400_1",
            "validator": "Nom Prenom",
            "validator_code": "login",
            "operator": "OPE",
            "comments": [],
            "type": "NM",
            "unit": None,
        }
    ]
    message = self.result(worklist=worklist)
    expected = """
    H|~^\\&|||SENDER||ORU|||RECEIVER||P|H2.1~L|20130715141912
    P|1|L036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||0123456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58|Diagnostic du SIL|MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
    OBR|1|12345678911~78910|12345678911~78910|T1|R|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||||||||||||OPE
    OBX|1|NM|T1||11||||||P|||||login~Nom Prenom||Advia2400_1
    L|1||1|5"""
    assert message == expected


def test_encode_result_status_results_clarilab(self):
    worklist = copy.deepcopy(self.result_worklist)
    worklist.samples[0].sample_id = "123456789"
    worklist.samples[0].results_status = "F"
    worklist.samples[0].results = [
        {
            "status": "P",
            "code": "T1",
            "lots": [],
            "value": [{"codification": "", "value": "11", "label": ""}],
            "instrument": "Advia2400_1",
            "validator_code": "login",
            "comments": [],
            "type": "NM",
            "unit": None,
        }
    ]
    message = self.result(worklist=worklist)
    expected = """
    H|~^\\&|||SENDER||ORU|||RECEIVER||P|H2.1~L|20130715141912
    P|1|L036666666||ADM_CLI|NYME~ANO~~~Mme|NJF|20110513|F||45 rue lambda~lala~Toulouse~~31000~FR||0123456789|ATTENDING~LAGRANGE&Joseph-Louis~L|||1.70|58|Diagnostic du SIL|MEDOC1^MEDOC2||||20130511~20130514||42~237~LOC
    OBR|1|123456789~78910|123456789~78910|T1|R|20130603110342|20130513114242|20130513124242||SAMPLER~Sampler~L|N|||||PRESCRIBER~Prescriber~~SER~SERVICE|0123^456||||||||F||||||||||OPE
    OBX|1|NM|T1||11||||||P|||||||Advia2400_1
    L|1||1|5
    """
    assert message == expected
