# -*- coding: utf8 -*-
from __future__ import unicode_literals

parser_class = "illumina"


def test_decode_result_bad_separateur(self):
    self.configure(illumina_separateur=";")
    worklist = self.decode(
        """
    batch_name  sample_barcode  sample_type sex_chrom   screen_type flowcell    class_sx    class_auto  anomaly_description qc_flag qc_reason   ff
    CFN141VCACXX    MultCNVs_Chr15_Chr16_14FF_6M_Itr2381    Singleton   yes genomewide  CFN141VCACXX    NO ANOMALY DETECTED - XY    ANOMALY DETECTED    DETECTED: dup(15)(q14q21.3);del(16)(q12.1q23.2) WARNING MULTIPLE ANOMALIES DETECTED 17%
    CFN141VCACXX    MultCNVs_Chr6_Chr6_WholeChr18_11FF_10M_Itr2395  Singleton   yes genomewide  CFN141VCACXX    INVALIDATED INVALIDATED INVALIDATED FAIL    DATA OUTSIDE OF EXPECTED RANGE  INVALIDATED
    CFN141VCACXX    XXEuploid_11FF_9M_Itr2431   Singleton   yes genomewide  CFN141VCACXX    NO ANOMALY DETECTED - XX    NO ANOMALY DETECTED NO ANOMALY DETECTED PASS    NONE    10%
    CFN141VCACXX    XXEuploid_8FF_4M_Itr2432    Singleton   yes genomewide  CFN141VCACXX    NO ANOMALY DETECTED - XX    NO ANOMALY DETECTED NO ANOMALY DETECTED PASS    NONE    8%
    """
    )
    expected = {"context": "result", "samples": []}
    assert worklist == expected


def test_decode_result(self):
    self.configure(illumina_separateur="\t")
    worklist = self.decode(
        """
    batch_name	sample_barcode	sample_type	sex_chrom	screen_type	flowcell	class_sx	class_auto	anomaly_description	qc_flag	qc_reason	ff
    CFN141VCACXX	MultCNVs_Chr15_Chr16_14FF_6M_Itr2381	Singleton	yes	genomewide	CFN141VCACXX	NO ANOMALY DETECTED - XY	ANOMALY DETECTED	DETECTED: dup(15)(q14q21.3);del(16)(q12.1q23.2)	WARNING	MULTIPLE ANOMALIES DETECTED	17%
    CFN141VCACXX	MultCNVs_Chr6_Chr6_WholeChr18_11FF_10M_Itr2395	Singleton	yes	genomewide	CFN141VCACXX	INVALIDATED	INVALIDATED	INVALIDATED	FAIL	DATA OUTSIDE OF EXPECTED RANGE	INVALIDATED
    CFN141VCACXX	XXEuploid_11FF_9M_Itr2431	Singleton	yes	genomewide	CFN141VCACXX	NO ANOMALY DETECTED - XX	NO ANOMALY DETECTED	NO ANOMALY DETECTED	PASS	NONE	10%
    CFN141VCACXX	XXEuploid_8FF_4M_Itr2432	Singleton	yes	genomewide	CFN141VCACXX	NO ANOMALY DETECTED - XX	NO ANOMALY DETECTED	NO ANOMALY DETECTED	PASS	NONE	8%
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "sex_chrom", "value": "yes"},
                    {"code": "screen_type", "value": "genomewide"},
                    {"code": "flowcell", "value": "CFN141VCACXX"},
                    {"code": "class_sx", "value": "NO ANOMALY DETECTED - XY"},
                    {"code": "class_auto", "value": "ANOMALY DETECTED"},
                    {
                        "code": "anomaly_description",
                        "value": "DETECTED: dup(15)(q14q21.3);del(16)(q12.1q23.2)",
                    },
                    {"code": "qc_flag", "value": "WARNING"},
                    {"code": "qc_reason", "value": "MULTIPLE ANOMALIES DETECTED"},
                    {"code": "ff", "value": "17"},
                ],
                "sample_id": "MultCNVs_Chr15_Chr16_14FF_6M_Itr2381",
                "type": "patient",
            },
            {
                "results": [
                    {"code": "sex_chrom", "value": "yes"},
                    {"code": "screen_type", "value": "genomewide"},
                    {"code": "flowcell", "value": "CFN141VCACXX"},
                    {"code": "class_sx", "value": "INVALIDATED"},
                    {"code": "class_auto", "value": "INVALIDATED"},
                    {"code": "anomaly_description", "value": "INVALIDATED"},
                    {"code": "qc_flag", "value": "FAIL"},
                    {"code": "qc_reason", "value": "DATA OUTSIDE OF EXPECTED RANGE"},
                    {"code": "ff", "value": "INVALIDATED"},
                ],
                "sample_id": "MultCNVs_Chr6_Chr6_WholeChr18_11FF_10M_Itr2395",
                "type": "patient",
            },
            {
                "results": [
                    {"code": "sex_chrom", "value": "yes"},
                    {"code": "screen_type", "value": "genomewide"},
                    {"code": "flowcell", "value": "CFN141VCACXX"},
                    {"code": "class_sx", "value": "NO ANOMALY DETECTED - XX"},
                    {"code": "class_auto", "value": "NO ANOMALY DETECTED"},
                    {"code": "anomaly_description", "value": "NO ANOMALY DETECTED"},
                    {"code": "qc_flag", "value": "PASS"},
                    {"code": "qc_reason", "value": "NONE"},
                    {"code": "ff", "value": "10"},
                ],
                "sample_id": "XXEuploid_11FF_9M_Itr2431",
                "type": "patient",
            },
            {
                "results": [
                    {"code": "sex_chrom", "value": "yes"},
                    {"code": "screen_type", "value": "genomewide"},
                    {"code": "flowcell", "value": "CFN141VCACXX"},
                    {"code": "class_sx", "value": "NO ANOMALY DETECTED - XX"},
                    {"code": "class_auto", "value": "NO ANOMALY DETECTED"},
                    {"code": "anomaly_description", "value": "NO ANOMALY DETECTED"},
                    {"code": "qc_flag", "value": "PASS"},
                    {"code": "qc_reason", "value": "NONE"},
                    {"code": "ff", "value": "8"},
                ],
                "sample_id": "XXEuploid_8FF_4M_Itr2432",
                "type": "patient",
            },
        ],
    }
    assert worklist == expected
