# -*- coding: utf8 -*-

from __future__ import unicode_literals

import datetime

from driverng import Worklist

parser_class = "kryptor"


def test_query(self):
    result = self.decode(
        """
    H|\\^&|||KRYPTOR^AUTOMATE KRYPTOR|||||LIS||P|1|20140213111038
    Q|1|^SAMPLE1|||ALL||||||||O
    L|1|F"""
    )
    expected = {
        "context": "query",
        "samples": [{"sample_id": "SAMPLE1", "type": "patient"}],
    }
    assert result == expected


def test_void_order(self):
    result = self.order(
        worklist=Worklist(context="order"),
        context={
            "query": self.parse(
                """
    H|\\^&|||KRYPTOR^AUTOMATE KRYPTOR|||||LIS||P|1|20140213111038
    Q|1|^SAMPLE1|||ALL||||||||O
    L|1|F"""
            )
        },
    )
    expected = """
    H|\\^&||||||||||P|1|20130715141912
    P|1
    O|1|SAMPLE1|||||||||||||||||||||||Z
    L|1|F"""
    assert result == expected


def test_order(self):
    result = self.order()
    expected = """H|\\^&||||||||||P|1|20130715141912
        P|1|036666666|||NYME^ANO|||F
        O|1|123456||^^^A^^1\\^^^B^^1|A||||||A||||||||||||||O
        L|1|F"""
    assert result == expected


def test_result(self):
    result = self.decode(
        """
    H|\\^&|||KRYPTOR^AUTOMATE KRYPTOR|||||LIS||P|1|201402061555
    P|1|123456|||TEST^ANONYME|||M
    O|1|20063207||^^^PCT^^1|||||||A||||||||^||||||F
    R|1|^^^PCT^^1^^F|3.669|µg/L||H||F|||20140206152700|20140206154700|A25075004049
    C|1|l|40\\46\\49
    C|2|l|42~32
    L|1|F"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "20063207",
                "rack": None,
                "results": [
                    {
                        "date": datetime.datetime(2014, 2, 6, 15, 47),
                        "flags": ["H", "40", "46", "49", "42", "32"],
                        "value": "3.669",
                        "code": "PCT",
                        "lots": [
                            {
                                "lot": "A25075004049",
                                "name": "A25075004049",
                                "type": "reagent",
                            }
                        ],
                    }
                ],
            }
        ],
    }
    assert result == expected


def test_qc(self):
    result = self.decode(
        """
    H|\\^&|||KRYPTOR^AUTOMATE KRYPTOR|||||LIS||P|1|201402061555
    P|1|||||||U
    O|1|COPC2010^^||^^^PCT^^1|||||||A||||||||^||||||F
    R|1|^^^PCT^^1^^F|3.669|µg/L||H||F|||20140206152700|20140206154700|A25075004049
    L|1|F"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "sample_id": "COPC2010",
                "control_lot": "COPC2010",
                "results": [
                    {
                        "flags": ["H"],
                        "date": datetime.datetime(2014, 2, 6, 15, 47),
                        "value": "3.669",
                        "code": "PCT",
                        "lots": [
                            {
                                "lot": "A25075004049",
                                "name": "A25075004049",
                                "type": "control",
                            }
                        ],
                    }
                ],
            }
        ],
    }
    assert result == expected
