# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime


parser_class = "mpl"


def test_decode_result(self):
    self.configure(mpl_traiter_lots_reactifs=True)
    worklist = self.decode(
        """
    H|^~\\&||||MPL||ORU|||SIL|||A2.2|
    P|1|03042400217203|||KARAM^NADIR||20071208|M|||||||||||||||201304241053
    OBR|1|140005225121^||LY%|R|201304241053|||||N|||||||||||
    OBX|10|NM|LY%^Lympho%||52.5|||C~e|||F|201304241426|23|||cb^T|
    C|1|L|^ATYLYM|
    M|1|L|R1^675846
    M|2|L|R2^675846
    P|1|03042400217203|||KARAM^NADIR||20071208|M|||||||||||||||201304241053
    OBR|1|140005225121^||CRP|R|201304241053|||||N|||||||||||
    OBX|10|NM|CRP||42||||||F|201304241426|24
    L|||1|6|"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "140005225121",
                "type": "patient",
                "results": [
                    {
                        "code": "LY%",
                        "comments": ["ATYLYM"],
                        "date": datetime.datetime(2013, 4, 24, 14, 26),
                        "flags": ["C", "e"],
                        "operator": "cb",
                        "instrument": "23",
                        "lots": [{"lot": "675846", "type": "reagent"}],
                        "value": "52.5",
                        "dilution": "",
                    },
                    {
                        "code": "CRP",
                        "comments": [],
                        "date": datetime.datetime(2013, 4, 24, 14, 26),
                        "flags": [],
                        "operator": "",
                        "instrument": "24",
                        "value": "42",
                        "dilution": "",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_version_5_05(self):
    self.configure(mpl_version_5_05=True, mpl_traiter_lots_reactifs=True)
    worklist = copy.deepcopy(self.order_worklist)
    worklist = self.decode(
        """
    H|^~\\&||||mpl||ORU|||pgp|||A2.2|
    P|1|061123001|061123001||TEST^ANONYME||19380817|F|||||||||||||||201801101040|
    OBR|1|0923001^061123001||B12~FOL~IGA~IGG~IGM~TSH~GROUPE|R|201801101040|||||N|||||||TEST|
    OBX|1|NM|B12||363.4|||H|||F|201801101319|6|||Roche|17^2
    M|1|L|R1^675846
    OBX|2|NM|FOL^10||11.85||||||F|201801101319|6|||Roche|17^2
    OBX|3|NM|IGA||1.27||||||F|201801101307|6|||Roche|17^2
    OBX|4|NM|IGG||8.66||||||F|201801101311|6|||Roche|17^2
    OBX|5|NM|IGM||0.98||||||F|201801101308|6|||Roche|17^2
    L|||1|14|
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "0923001",
                "type": "patient",
                "rack": "17-2",
                "results": [
                    {
                        "code": "B12",
                        "value": "363.4",
                        "flags": ["H"],
                        "comments": [],
                        "dilution": "",
                        "date": datetime.datetime(2018, 1, 10, 13, 19),
                        "operator": "Roche",
                        "instrument": "6",
                        "lots": [{"lot": "675846", "type": "reagent"}],
                    },
                    {
                        "code": "FOL",
                        "value": "11.85",
                        "flags": [],
                        "comments": [],
                        "dilution": "10",
                        "date": datetime.datetime(2018, 1, 10, 13, 19),
                        "operator": "Roche",
                        "instrument": "6",
                    },
                    {
                        "code": "IGA",
                        "value": "1.27",
                        "flags": [],
                        "comments": [],
                        "dilution": "",
                        "date": datetime.datetime(2018, 1, 10, 13, 7),
                        "operator": "Roche",
                        "instrument": "6",
                    },
                    {
                        "code": "IGG",
                        "value": "8.66",
                        "flags": [],
                        "comments": [],
                        "dilution": "",
                        "date": datetime.datetime(2018, 1, 10, 13, 11),
                        "operator": "Roche",
                        "instrument": "6",
                    },
                    {
                        "code": "IGM",
                        "value": "0.98",
                        "flags": [],
                        "comments": [],
                        "dilution": "",
                        "date": datetime.datetime(2018, 1, 10, 13, 8),
                        "operator": "Roche",
                        "instrument": "6",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_comment_C_4_1(self):
    worklist = self.decode(
        """
    H|^~\\&||||MPL||ORU|||SIL|||A2.2|
    P|1|03042400217203|||KARAM^NADIR||20071208|M|||||||||||||||201304241053
    OBR|1|1310001469^1310001469||RK~RNA|R|201604211515|201604211509||||N|||201604211511|Sérum|CLARISYS|Laboratoire|AREVA
    OBX|1|ST|RK^K||<10||||||F|201604211513||||ble^T
    C|1|L|hello|
    L|||1|6|"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "1310001469",
                "type": "patient",
                "results": [
                    {
                        "code": "RK",
                        "comments": ["hello"],
                        "date": datetime.datetime(2016, 4, 21, 15, 13),
                        "flags": [],
                        "dilution": "",
                        "operator": "ble",
                        "instrument": "",
                        "value": "<10",
                    }
                ],
            }
        ],
    }
    assert worklist == expected


def test_encode_order(self):
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0].results.add(code="POID", value="100", anterior=False)
    message = self.order(worklist=worklist)
    expected = """
    H|^~\\&|||MCA||ORM||||||A2.2|20130715141912
    P|1|036666666|||NYME^ANO|NJF|20110513|F
    C|1|L|Commentaire patient no1
    OBR|1|123456||B|A||20130513114242||||N|||20130603110342||PRESCRIBER||SER
    C|1|L|test retour chariot
    OBX|1|NM|A||42||||||A
    OBX|2|NM|B||20130213||||||A|20110506121300
    OBX|3|NM|POID||100||||||A
    OBR|2|123456||C|A||20130513114242||||C|||20130603110342||PRESCRIBER||SER
    C|1|L|test retour chariot
    L|1||1|11
    """
    assert message == expected
