# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "stago"


def test_order(self):
    result = self.order()
    expected = """H|\\^&|||2^2.00|||||||P|1.00|20130715141912
        P|1|||NYME^ANO^2 ans^F|||20110513
        O|1|123456||^^^A\\^^^B|S
        L|1|N"""
    assert result == expected


def test_result(self):
    date = datetime.datetime(2012, 6, 13, 15, 36, 25)
    result = self.decode(
        """
    H|\\^&|||99^2.10|||||||P|1.00|20120613153625
    P|1|||ANO^NYME^54 ans^M|||
    O|1|140001708831^357568^3|||R
    R|1|^^^1|30|%||||F||||20120613153625
    M|1|1|C\\H||11488^BUF
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "140001708831",
                "type": "patient",
                "rack": "357568-3",
                "results": [
                    {"flags": ["C", "H", "1"], "date": date, "value": "30", "code": "1"}
                ],
            }
        ],
    }
    assert result == expected


def test_result_qc(self):
    self.configure(qc_prefix="99")
    date = datetime.datetime(2012, 6, 13, 15, 36, 25)
    result = self.decode(
        """
    H|\\^&|||99^2.10|||||||P|1.00|20120613153625
    P|1|||ANO^NYME^54 ans^M|||
    O|1|990001708831^357568^3|||R
    R|1|^^^1|30|%||||F||||20120613153625
    M|1|A|@
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "990001708831",
                "type": "control",
                "control_lot": "990001708831",
                "control_level": "990001708831",
                "control_name": "",
                "results": [{"flags": ["@"], "date": date, "value": "30", "code": "1"}],
            }
        ],
    }
    assert result == expected


def test_qc_niveau_lot(self):
    result = self.decode(
        """
    H|\\^&|||99^2.10|||||||Q|1.00|20140306081242
    P|1|||^^^|||
    O|1|PBQ1^^^006Y13|||R
    R|1|^^^5|33.3|Sec.||||F||||20140305184859
    M|1|A|@
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "PBQ1006Y13",
                "type": "control",
                "control_lot": "PBQ1006Y13",
                "control_name": "006Y13",
                "control_level": "PBQ1",
                "results": [
                    {
                        "flags": ["@"],
                        "date": datetime.datetime(2014, 3, 5, 18, 48, 59),
                        "code": "5",
                        "value": "33.3",
                    }
                ],
            }
        ],
    }
    assert result == expected
