# -*- coding: utf8 -*-
from __future__ import unicode_literals

from datetime import datetime

parser_class = "zenit"


def test_order(self):
    result = self.order(context={"control_id": "1374155237204"})
    expected = """
        MSH|^~\\&|MCA^HOST||S02^ZENIT||20130715141912||OML^021^OML_021|1374155237204|P|2.5
        PID|1|036666666|||NYME^ANO||20110513|F
        PV1|1|N|LAB
        ORC|NW|||||N|||20130715141912||||||20130715141912
        OBR|1|123456||A||20130715141912
        OBR|2|123456||B||20130715141912"""
    assert result == expected


def test_result(self):
    result = self.decode(
        r"""
    MSH|^~\&|S02^ZENIT||LIS^HOST||20060627121511||ORU^R01^ORU_R01|00000001151316898736|P|2.5
    PID|0001||||AZZURRI^LUCIANO^^^^||19400123|M
    OBR|0001|2006060001||Ema4^Ema4^Zen-IT||20060626000000|20060626000000
    OBX|0001|CE|Ema4^Ema4^Zen-IT|1|POS^Positive^Zen-IT|||A|||F|||20060226181135||ziadmin
    OBX|0002|ST|Ema4^Ema4^Zen-IT|2|1:250||||||F|||20060226181135||ziadmin
    OBX|0003|ST|Ema4^Ema4^Zen-IT|3|Fine Speckled||||||F|||20060226181135||ziadmin
    OBX|0004|ST|Ema4^Ema4^Zen-IT|99|No other significant information.||||||F|||20060226181135||ziadmin
    OBR|0002|2006060001||Anca6^Anca6^Zen-IT||20060626000000|20060626000000
    OBX|0001|CE|Anca6^Anca6^Zen-IT|1|NEG^Negative^Zen-IT|||N|||F|||20060226181135||ziadmin
    OBX|0002|ST|Anca6^Anca6^Zen-IT|99|||||||F|||20060226181135||ziadmin
    OBR|0003|2006060001||ACAIGA^ACA IgA^Zen-IT||20060626000000|20060626000000
    OBX|0001|NM|ACAIGA^ACA IgA^Zen-IT||2.5|||N|||F|||20060226181135||ziadmin
    OBR|0004|2006060001||ACAIGG^ACA IgG^Zen-IT|R|20060626000000|20060626000000
    OBX|0001|NM|ACAIGG^ACA IgG^Zen-IT||65.3|||N|||F|||20060226181135||ziadmin
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "2006060001",
                "type": "patient",
                "results": [
                    {
                        "code": "Ema4",
                        "date": datetime(2006, 2, 26, 18, 11, 35),
                        "flags": ["A"],
                        "value": "POS",
                    },
                    {
                        "code": "Ema4",
                        "date": datetime(2006, 2, 26, 18, 11, 35),
                        "flags": [],
                        "value": "1:250",
                    },
                    {
                        "code": "Ema4",
                        "date": datetime(2006, 2, 26, 18, 11, 35),
                        "flags": [],
                        "value": "Fine Speckled",
                    },
                    {
                        "code": "Ema4",
                        "date": datetime(2006, 2, 26, 18, 11, 35),
                        "flags": [],
                        "value": "No other significant information.",
                    },
                    {
                        "code": "Anca6",
                        "date": datetime(2006, 2, 26, 18, 11, 35),
                        "flags": ["N"],
                        "value": "NEG",
                    },
                    {
                        "code": "Anca6",
                        "date": datetime(2006, 2, 26, 18, 11, 35),
                        "flags": [],
                        "value": "",
                    },
                    {
                        "code": "ACAIGA",
                        "date": datetime(2006, 2, 26, 18, 11, 35),
                        "flags": ["N"],
                        "value": "2.5",
                    },
                    {
                        "code": "ACAIGG",
                        "date": datetime(2006, 2, 26, 18, 11, 35),
                        "flags": ["N"],
                        "value": "65.3",
                    },
                ],
            }
        ],
    }
    assert result == expected
